<#-- START TEMPLATE HEADER -->
<#-- category=appservers/weblogic/7.0 -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=build.xml -->
<#-- relativeOutputDirectory=ant/weblogic-7.0 -->
<#-- END TEMPLATE HEADER -->
<?xml version="1.0" encoding="ISO-8859-1" ?>

<!--

  Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.

  Modification and redistribution of this file is permitted subject to the
  terms and conditions of the RapidJ End User License Agreement:
  http://www.codecanvas.com.au/rapidj/license.php

  {{$RapidJ$}}

-->

<!--

${f.formatLines(project.copyright, '  ', 80)}

-->

<project name="${project.name?xml}" default="compile" basedir=".">
<#noparse>

<!-- =============== Customisable Build Property Definitions ============== -->

<!--

  The following properties can be customised either here or by defining them
  in the build.properties file.

  NOTE: Properties are immutable. Once they have been set for the first time
  using the <property> task they cannot be changed.

  app.name             The name of the application being built. Is used when
                       creating directories and files related to the build
                       process. Defaults to "myapp".

  app.version          The version of the application being built.

  build.dir.name       The name of the directory under the project directory
                       used to construct the exploded ear file. Defaults to
                       "build".

  dist.dir.name        The name of the directory under the project directory
                       used to store the application ear file. Defaults to
                       "dist".

  javadoc.dir.name     The name of the directory under the project directory
                       used to store the javadoc generated by the javadoc
                       target. Defaults to "javadoc".

  app.path             The url that the application should be installed to.
                       Defaults to "/${app.name}".

  weblogic.home        The location of the Weblogic installation, used to add
                       Weblogic libraries to the compile classpath.

  domain.home          The location of the Weblogic domain that the application
                       is installed to.

   {{$RapidJ$}}

-->

<!-- Load properties from build.properties -->
  <property file="build.properties"/>

<!-- Set default values for applicable unset properties -->
  <property name="app.name"            value="myapp"/>
  <property name="app.version"         value="1"/>
  <property name="build.dir.name"      value="build"/>
  <property name="dist.dir.name"       value="dist"/>
  <property name="javadoc.dir.name"    value="javadoc"/>


<!-- ===================== Build Property Definitions ========================= -->

<!--

  The following properties cannot be customised without making changes to the
  rest of the script.

-->

  <property name="project.dir"     value="${basedir}/../../"/>
  <property name="build.dir"       value="${project.dir}/${build.dir.name}"/>
  <property name="dist.dir"        value="${project.dir}/${dist.dir.name}"/>
  <property name="javadoc.dir"     value="${project.dir}/${javadoc.dir.name}"/>
  <property name="src.dir"         value="${project.dir}/src"/>
  <property name="web.dir"         value="${project.dir}/web"/>


<!--  ==================== Compilation Control Options ==================== -->

<!--

  Property definitions for options passed to javac in the "compile" target.

  compile.debug        Should compilation include the debug option?

  compile.deprecation  Should compilation include the deprecation option?

  compile.optimize     Should compilation include the optimize option?

-->

  <property name="compile.debug"         value="true"/>
  <property name="compile.deprecation"   value="false"/>
  <property name="compile.optimize"      value="false"/>


<!-- ==================== External Dependencies =========================== -->

<!--

  Property definitions that define the location of external JAR files on which
  the application depends. Generally these JAR files are used for two purposes.
  1) To compile the application source files, and
  2) To be included in the application WAR file in the WEB-INF/lib directory.

-->

  <property name="struts.jar"             value="../../lib/struts.jar"/>
  <property name="commons-beanutils.jar"  value="../../lib/commons-beanutils.jar"/>
  <property name="commons-digester.jar"   value="../../lib/commons-digester.jar"/>
  <property name="commons-logging.jar"    value="../../lib/commons-logging.jar"/>
  <property name="commons-validator.jar"  value="../../lib/commons-validator-1.1.4.jar"/>
  <property name="log4j.jar"              value="../../lib/log4j-1.2.9.jar"/>
  <property name="freemarker.jar"         value="../../lib/freemarker.jar"/>
  <property name="jakarta-oro.jar"        value="../../lib/jakarta-oro-2.0.8.jar"/>


<!-- ==================== Compilation Classpath =========================== -->

<!--

  Create the classpath that is required to compile the application.

-->

  <path id="compile.classpath">

    <!-- Include all JAR files required to compile the application -->
    <pathelement location="${j2ee.jar}"/>
    <pathelement location="${struts.jar}"/>
    <pathelement location="${commons-beanutils.jar}"/>
    <pathelement location="${commons-digester.jar}"/>
    <pathelement location="${commons-logging.jar}"/>
    <pathelement location="${commons-validator.jar}"/>
    <pathelement location="${log4j.jar}"/>
    <pathelement location="${freemarker.jar}"/>

    <!-- Include all classes that Weblogic exposes to applications -->
    <fileset dir="${weblogic.home}/server/lib">
      <include name="*.jar"/>
    </fileset>
    <fileset dir="${weblogic.home}/server/ext">
      <include name="*.jar"/>
    </fileset>

  </path>


<!-- ==================== Clean Target ==================================== -->

<!--

  Deletes the "build" and "dist" directories. This will ensure that the
  application will be built from scratch next time.

-->

  <target name="clean"
      description="Delete old build and dist directories">
    <delete dir="${build.dir}"/>
    <delete dir="${dist.dir}"/>
  </target>


<!-- ==================== Prepare Target ================================== -->

<!--

  Creates the main build related directories and copies in any static content
  required by the web application.

-->

  <target name="prepare"
      description="Create build directories and copy static content">

    <!-- Check that required properties are set -->
    <fail unless="dataSource.jdbcDriver"
        message="dataSource.jdbcDriver not specified in build.properties"/>
    <fail unless="dataSource.jdbcUrl"
        message="dataSource.jdbcUrl not specified in build.properties"/>
    <fail unless="dataSource.username"
        message="dataSource.username not specified in build.properties"/>
    <fail unless="dataSource.password"
        message="dataSource.password not specified in build.properties"/>

    <!-- Create build directories as needed -->
    <mkdir dir="${build.dir}"/>
    <mkdir dir="${build.dir}/WEB-INF"/>
    <mkdir dir="${build.dir}/META-INF"/>
    <mkdir dir="${build.dir}/WEB-INF/classes"/>

    <!-- Setup the filter used when copying files -->
    <filter filtersfile="build.properties"/>

    <!-- Copy the web application static content -->
    <!-- Note: Weblogic 7 does not allow RequestDispatcher access to resources -->
    <!-- under WEB-INF so need to move the JSPs into the root of the web dir. -->
    <copy todir="${build.dir}">
      <fileset dir="${web.dir}" excludes="WEB-INF/web.xml,WEB-INF/jsp,**/*.shd"/>
      <fileset dir="${web.dir}/WEB-INF" includes="jsp/*"/>
    </copy>

    <!-- Copy (with filtering) web.xml -->
    <copy    file="${web.dir}/WEB-INF/web.xml"
            todir="${build.dir}/WEB-INF"
        filtering="true"
        overwrite="true"/>

    <!-- Copy (with filtering) weblogic.xml -->
    <copy   file="weblogic.xml"
           todir="${build.dir}/WEB-INF"
        filtering="true"
        overwrite="true"/>

    <!-- Copy (with filtering) application.xml -->
    <copy   file="application.xml"
           todir="${build.dir}/META-INF"
        filtering="true"
        overwrite="true"/>

    <!-- Copy (with filtering) weblogic-application.xml -->
    <copy   file="weblogic-application.xml"
           todir="${build.dir}/META-INF"
        filtering="true"
        overwrite="true"/>

    <!-- Copy application resources -->
    <copy  todir="${build.dir}/WEB-INF/classes">
      <fileset dir="${src.dir}"
          excludes="application.properties,**/*.java,**/*.shd"/>
    </copy>

    <!-- Copy (with filtering) application.properties -->
    <copy   file="${src.dir}/application.properties"
           todir="${build.dir}/WEB-INF/classes/"
        filtering="true"
        overwrite="true"/>

    <!-- Copy external dependencies as required -->
    <mkdir  dir="${build.dir}/WEB-INF/lib"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${struts.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-beanutils.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-digester.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-logging.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-validator.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${log4j.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${freemarker.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${jakarta-oro.jar}"/>

  </target>


<!-- ==================== Compile Target ================================== -->

<!--

  Compiles the Java source files in ${src.dir} to the WEB-INF/classes directory.

-->

  <target name="compile" depends="prepare"
      description="Compile Java sources">

    <!-- Compile Java classes as necessary -->
    <javac srcdir="${src.dir}"
          destdir="${build.dir}/WEB-INF/classes"
            debug="${compile.debug}"
      deprecation="${compile.deprecation}"
         optimize="${compile.optimize}">
        <classpath refid="compile.classpath"/>
    </javac>

  </target>


<!-- ==================== Dist Target ===================================== -->

<!--

  Creates an EAR file containing the application.

-->

  <target name="dist" depends="compile"
      description="Create application EAR file">

	<!-- Create the dist directory as needed -->
	<mkdir dir="${dist.dir}"/>

    <!-- Increment the build number -->
    <buildnumber/>

    <!-- Create application WAR file -->
    <delete file="${app.name}.war"/>
    <jar jarfile="${build.dir}/${app.name}.war"
         basedir="${build.dir}"
        excludes="META-INF/*,${app.name}.war"/>

    <!-- Create application EAR file -->
    <jar jarfile="${dist.dir}/${app.name}.ear"
         basedir="${build.dir}"
        includes="META-INF/*,*.war">
      <manifest>
        <attribute name="Implementation-Version"
                  value="${app.version} build ${build.number}"/>
      </manifest>
    </jar>

  </target>


<!-- ==================== Install Target ================================== -->

<!--

  Installs the application EAR file into Weblogic.

  NOTE: This installs the application to the applications directory of the
  specified domain. This is a quick and easy way to install applications in
  development mode. For production installations please refer to:
  http://e-docs.bea.com/wls/docs81/deployment/overview.html

  NOTE: You do not need to "remove" before calling this.

-->

  <target name="install" depends="dist"
      description="Install application to Weblogic">

    <!-- Copy the application war to the specified domain -->
    <copy file="${dist.dir}/${app.name}.ear"
         toDir="${domain.home}/applications"/>

  </target>


<!-- ==================== Remove Target =================================== -->

<!--

  Removes the application from Weblogic.

-->

  <target name="remove"
      description="Remove application from Weblogic">

    <!-- Delete the application EAR file from the specified domain -->
    <delete file="${domain.home}/applications/${app.name}.ear"/>

  </target>


<!-- ==================== Javadoc Target ================================== -->

<!--

  Creates Javadoc from the application source files. The Javadoc is provided as
  a convenience for developers and it is not included in the application WAR
  file.

-->

  <target name="javadoc" depends="compile"
      description="Create Javadoc API documentation">

    <!-- Create the Javadoc directory as needed -->
    <mkdir dir="${javadoc.dir}/api"/>

    <!-- Create the Javadoc -->
    <javadoc sourcepath="${src.dir}"
                destdir="${javadoc.dir}/api"
           packagenames="*">
      <classpath refid="compile.classpath"/>
    </javadoc>

  </target>

</project>
</#noparse>
