<#-- START TEMPLATE HEADER -->
<#-- category=databases/hsqldb/1.7 -->
<#-- type=database -->
<#-- description=HSQLDB 1.7-->
<#-- version=1.0 -->
<#-- END TEMPLATE HEADER -->
------------------------------------------------------
-- HSQLDB 1.7 Database Script
------------------------------------------------------
<#if constraintsFlag && dropDeleteFlag>

------------------------------------------------------
-- Drop constraints
------------------------------------------------------
<#list entities as entity>

--
-- Drop foreign keys for table '${f.tab(entity.name)}'
--

	<#list entity.relationships as relationship>
		<#if relationship.child == entity>
			<#-- Parent relationship -->
			<#if relationship.identifying>
ALTER TABLE ${f.tab(entity.name)} DROP CONSTRAINT ${f.constraintName(f.tab(entity.name),f.col(relationship.parentAssociationName)+'_id','fk',30)};
	 		</#if>
	 	<#else>
	 		<#-- Child relationship -->
	 		<#if !relationship.identifying>
ALTER TABLE ${f.tab(entity.name)} DROP CONSTRAINT ${f.constraintName(f.tab(entity.name),f.col(relationship.childAssociationName)+'_id','fk',30)};
	 		</#if>
	 	</#if>
	</#list>
</#list>
</#if>
<#if createTablesFlag && dropDeleteFlag>

------------------------------------------------------
-- Drop tables
------------------------------------------------------

<#list entities as entity>
DROP TABLE ${f.tab(entity.name)} IF EXISTS;
</#list>
</#if>
<#if createTablesFlag>

------------------------------------------------------
-- Create tables
------------------------------------------------------
<#list entities as entity>

--
-- Create table '${f.tab(entity.name)}'
--

CREATE TABLE ${f.tab(entity.name)}
	(${f.col(entity.name)}_id BIGINT NOT NULL,
	<#list entity.relationships as relationship>
		<#if relationship.child == entity>
			<#-- Parent relationship -->
			<#if relationship.identifying>
	 ${f.col(relationship.parentAssociationName)}_id BIGINT NOT NULL,
	 		</#if>
	 	<#else>
	 		<#-- Child relationship -->
	 		<#if !relationship.identifying>
	 ${f.col(relationship.childAssociationName)}_id BIGINT,
	 		</#if>
	 	</#if>
	</#list>
	<#list entity.attributes as attribute>
	 ${f.col(attribute.name)} <#rt>
		<#if attribute.javaType='java.lang.Boolean'><#t>BOOLEAN
		<#elseif attribute.javaType='java.lang.Byte'><#t>SMALLINT
		<#elseif attribute.javaType='java.lang.Double'><#t>DOUBLE PRECISION
		<#elseif attribute.javaType='java.lang.Float'><#t>FLOAT
		<#elseif attribute.javaType='java.lang.Integer'><#t>INTEGER
		<#elseif attribute.javaType='java.lang.Long'><#t>BIGINT
		<#elseif attribute.javaType='java.lang.Short'><#t>SMALLINT
		<#elseif attribute.javaType='java.lang.String'><#t>VARCHAR(${attribute.size})
		<#elseif attribute.javaType='java.math.BigDecimal'><#t>DECIMAL(16,4)
		<#elseif attribute.javaType='java.sql.Blob'><#t>BLOB
		<#elseif attribute.javaType='java.sql.Clob'><#t>CLOB
		<#elseif attribute.javaType='java.sql.Date'><#t>DATE
		<#elseif attribute.javaType='java.sql.Time'><#t>TIME
		<#elseif attribute.javaType='java.sql.Timestamp'><#t>TIMESTAMP
		<#else><#t>UNKNOWN</#if><#rt>
		<#if attribute.required><#t> NOT NULL</#if>,
	</#list>
	 version BIGINT NOT NULL,
	 update_datetime TIMESTAMP NOT NULL,
	 update_uid VARCHAR(255) NOT NULL,
	 create_datetime TIMESTAMP NOT NULL,
	 create_uid VARCHAR(255) NOT NULL,
	 PRIMARY KEY (${f.col(entity.name)}_id)
	);
</#list>
</#if>
<#if constraintsFlag>

------------------------------------------------------
-- Constraints
------------------------------------------------------
<#list entities as entity>

--
-- Constraints for table '${f.tab(entity.name)}'
--
	<#list entity.relationships as relationship>
		<#if relationship.child == entity>
			<#-- Parent relationship -->
			<#if relationship.identifying>

CREATE INDEX ${f.constraintName(f.tab(entity.name),f.col(relationship.parentAssociationName)+'_id','idx',30)}
	ON ${f.tab(entity.name)} (${f.col(relationship.parentAssociationName)}_id);

ALTER TABLE ${f.tab(entity.name)}
	ADD CONSTRAINT ${f.constraintName(f.tab(entity.name),f.col(relationship.parentAssociationName)+'_id','fk',30)}
	FOREIGN KEY (${f.col(relationship.parentAssociationName)}_id)
	REFERENCES ${f.tab(relationship.parent.name)}(${f.col(relationship.parent.name)}_id);
				<#if relationship.isOneToOne()>

ALTER TABLE ${f.tab(entity.name)}
	ADD CONSTRAINT ${f.constraintName(f.tab(entity.name),f.col(relationship.parentAssociationName)+'_id','un',30)}
	UNIQUE (${f.col(relationship.parentAssociationName)}_id);
				</#if>
	 		</#if>
	 	<#else>
	 		<#-- Child relationship -->
	 		<#if !relationship.identifying>

CREATE INDEX ${f.constraintName(f.tab(entity.name),f.col(relationship.childAssociationName)+'_id','idx',30)}
	ON ${f.tab(entity.name)} (${f.col(relationship.childAssociationName)}_id);

ALTER TABLE ${f.tab(entity.name)}
	ADD CONSTRAINT ${f.constraintName(f.tab(entity.name),f.col(relationship.childAssociationName)+'_id','fk',30)}
	FOREIGN KEY (${f.col(relationship.childAssociationName)}_id)
	REFERENCES ${f.tab(relationship.child.name)}(${f.col(relationship.child.name)}_id);
	 		</#if>
	 	</#if>
	</#list>
</#list>
</#if>
<#if testRecordsFlag>

------------------------------------------------------
-- Test records
------------------------------------------------------
<#list entities as entity>

--
-- Test records for table '${f.tab(entity.name)}'
--

	<#if dropDeleteFlag>DELETE FROM ${f.tab(entity.name)} WHERE ${f.col(entity.name)}_id > 0 and ${f.col(entity.name)}_id <= ${entity.numTestRecordsActual?string("#")};</#if><#t>
	<#list 1..entity.numTestRecordsActual as i>
		<#t>INSERT INTO ${f.tab(entity.name)} (
		<#t>${f.col(entity.name)}_id
		<#list entity.relationships as relationship><#rt>
			<#if relationship.child == entity>
				<#-- Parent relationship -->
				<#t><#if relationship.identifying>,${f.col(relationship.parentAssociationName)}_id</#if>
		 	<#else>
	 			<#-- Child relationship -->
	 			<#t><#if !relationship.identifying>,${f.col(relationship.childAssociationName)}_id</#if>
		 	</#if>
		</#list>
		<#t><#list entity.attributes as attribute><#if attribute.testDataProducer?exists>,${f.col(attribute.name)}</#if></#list>
		<#t>,version,update_datetime,update_uid,create_datetime,create_uid
		<#t>) VALUES (
		<#t>${i?string("#")}
		<#list entity.relationships as relationship><#rt>
			<#if relationship.child == entity>
				<#-- Parent relationship -->
				<#t><#if relationship.identifying>,${((i-1) % relationship.parent.numTestRecordsActual + 1)?string("#")}</#if>
		 	<#else>
	 			<#-- Child relationship -->
	 			<#t><#if !relationship.identifying>,null</#if>
		 	</#if>
		</#list>
		<#list entity.attributes as attribute>
			<#if attribute.testDataProducer?exists><#t>,<#rt>
				<#if attribute.testDataProducer.hasMoreData(attribute)>
					<#if attribute.javaType='java.lang.Boolean'>
						<#if attribute.testDataProducer.getData()><#t>'true'<#else><#t>'false'</#if><#t>
					<#else>
						<#assign value = attribute.testDataProducer.getData()/>
						<#if     attribute.javaType='java.lang.Byte'><#t>${value?string("#")}
						<#elseif attribute.javaType='java.lang.Short'><#t>${value?string("#")}
						<#elseif attribute.javaType='java.lang.Integer'><#t>${value?string("#")}
						<#elseif attribute.javaType='java.lang.Long'><#t>${value?string("#")}
						<#elseif attribute.javaType='java.lang.Float'><#t>${value?string("#.####")}
						<#elseif attribute.javaType='java.lang.Double'><#t>${value?string("#.####")}
						<#elseif attribute.javaType='java.math.BigDecimal'><#t>${value?string("#.####")}
						<#elseif attribute.javaType='java.lang.String'><#t>'${value}'
						<#elseif attribute.javaType='java.sql.Date'><#t>'${value?string("yyyy-MM-dd")}'
						<#elseif attribute.javaType='java.sql.Time'><#t>'${value?string("HH:mm:ss")}'
						<#elseif attribute.javaType='java.sql.Timestamp'><#t>'${value?string("yyyy-MM-dd HH:mm:ss")}'
						<#elseif attribute.javaType='java.sql.Clob'><#t>'${value}'
						<#elseif attribute.javaType='java.sql.Blob'><#t>${value}
						<#else><#t>UNKNOWN</#if><#rt>
					</#if>
				<#else><#t>null
				</#if><#t>
			</#if><#t>
		</#list><#lt>
		<#lt>,1,CURRENT_TIMESTAMP,'test',CURRENT_TIMESTAMP,'test');
	</#list>
</#list>
<#list entities as entity>
	<#list entity.relationships as relationship><#rt>
		<#if relationship.child == entity>
			<#-- Parent relationship -->
			<#if !relationship.identifying>

--
-- Non identified child updates for table '${f.tab(relationship.parent.name)}'
--

				<#list 1..relationship.parent.numTestRecordsActual as i>
UPDATE ${f.tab(relationship.parent.name)} SET ${f.col(relationship.childAssociationName)}_id = ${((i-1)%entity.numTestRecordsActual + 1)?string("#")} WHERE ${f.col(relationship.parent.name)}_id = ${i?string("#")};
				</#list>
			</#if>
	 	</#if>
	</#list>
</#list>
</#if>
<#if supportTablesFlag>

------------------------------------------------------
-- Support tables
------------------------------------------------------
<#if dropDeleteFlag>

--
-- Drop table 'Sequence'
--

DROP TABLE Sequence IF EXISTS;
</#if>

--
-- Create table 'Sequence'
--

CREATE TABLE Sequence (
	name VARCHAR(100) NOT NULL,
	last_id INTEGER NOT NULL
);

--
-- Insert data into 'Sequence' table
--

<#list entities as entity>
INSERT INTO Sequence(name, last_id) VALUES ('${f.att(entity.name)}Id', ${entity.numTestRecordsActual?string("#")});
</#list>

--
-- Update data in 'Sequence' table
--

<#list entities as entity>
UPDATE Sequence SET last_id = (SELECT MAX(${f.col(entity.name)}_id) FROM ${f.tab(entity.name)}) WHERE name = '${f.att(entity.name)}Id';
</#list>
</#if>
