<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=ActionBase.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=web.action -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.web.action;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;

import ${project.rootPackage}.util.convert.ConvertUtils;
import ${project.rootPackage}.util.copy.CopyException;
import ${project.rootPackage}.util.copy.CopyUtils;

/**
 * <p>The abstract super class for all Action classes for the application.
 *    Provides subclasses with the following functionality:</p>
 *
 * <ul>
 *   <li>Dynamic dispatching based on the "dispatch" request parameter</li>
 *   <li>Ability to easily add messages and errors to the request</li>
 * </ul>
 */
public abstract class ActionBase extends Action {

	/**
	 * Log4J Logger for this class
	 */
	public static final Logger logger = Logger.getLogger(ActionBase.class);

	/**
	 * Cache of Methods for the dispatches
	 */
	private Map executeMethods = new HashMap();

	/**
	 * Reference to the ConvertUtils instance used by the copy method.
	 */
	private ConvertUtils convertUtils = null;

	/**
	 * <p>The main entry point that dynamically delegates to another method.
	 *
	 * <p>The name of the delegated method is determined by the "dispatch"
	 * request parameter. For example:
	 *
	 * <table>
	 *   <tr>
	 *     <th>Dispatch</th>
	 *     <th>Method Name</th>
	 *   </tr>
	 *   <tr>
	 *     <td>null</td>
	 *     <td>executeDefault</td>
	 *   </tr>
	 *   <tr>
	 *     <td>savePerson</td>
	 *     <td>executeSavePerson</td>
	 *   </tr>
	 * </table>
	 *
	 * <p>Once the name of the method has been determined, an attempt is
	 * made to locate a method with this name and a one of the following
	 * method signatures.
	 *
	 * <ol>
	 *   <li>executeXXXX(Request, form.getClass())</li>
	 *   <li>executeXXXX(Request, ActionForm)</li>
	 *   <li>executeXXXX(Request)</li>
	 * </ol>
	 *
	 * <p> The execute method can return an ActionForward or void. If it
	 * returns void then the "success" ActionForward will be returned by
	 * this method.
	 */
	public ActionForward execute(ActionMapping mapping,
								 ActionForm form,
								 HttpServletRequest request,
								 HttpServletResponse response)
								 throws Exception {

		// Save the start time in the request
		request.setAttribute("start", new Long(System.currentTimeMillis()));

		// Debug the request
		if (logger.isDebugEnabled()) {
			logger.debug("--------------- Start Request Debug ----------------------------");
			logger.debug("Parameters:");
			Enumeration enum = request.getParameterNames();
			while (enum.hasMoreElements()) {
				String name = (String) enum.nextElement();
				logger.debug("\t"+name+"='"+request.getParameter(name)+"'");
			}
			Cookie[] cookies = request.getCookies();
			logger.debug("Cookies:");
			if (cookies!=null) {
				for (int i = 0; i < cookies.length; i++) {
					logger.debug("\t"+cookies[i].getName()+"='"+cookies[i]+"'");
				}
			}
			logger.debug("Session Parameters:");
			HttpSession session = request.getSession(false);
			if (session!=null) {
				enum = session.getAttributeNames();
				while (enum.hasMoreElements()) {
					String name = (String) enum.nextElement();
					logger.debug("\t"+name+"='"+session.getAttribute(name)+"'");
				}
			}
			logger.debug("--------------- End Request Debug ------------------------------");
		}

		// Construct the Request object
		Request req = new Request(mapping, form, request, response);

		// Call the pre execution method
		preExecute(req);

		// Get the execute method based on the dispatch
		String dispatch = request.getParameter("dispatch");
		Method method = getExecuteMethod(dispatch, form);
		ActionForward forward = null;

		// Invoke the execute method with the correct parameters
		try {
			if (method.getParameterTypes().length==1) {
				forward = (ActionForward) method.invoke(this, new Object[]{req});
			} else {
				forward = (ActionForward) method.invoke(this, new Object[]{req,form});
			}
		} catch (InvocationTargetException ite) {
			// Extract and throw the cause exception
			Throwable target = ite.getTargetException();
			logger.error("exception invoking Actions execute method", ite);
			if (target instanceof Exception) {
				throw (Exception) target;
			} else {
				throw new Exception(target.getMessage());
			}
		}

		// If the method returned a void, then use the "success" forward
		if (method.getReturnType() == Void.TYPE) {
			forward = (req.getMapping().findForward("success"));
		}
		logger.debug("using forward=" + forward);

		// Call the postExecute method.
		postExecute(req);

		// Return the forward. This may be null which will indicate to
		// Struts that the request has been processed and that nothing
		// else needs to be done.
		return forward;

	}

	/**
	 * This method returns the method that should be delegated to for the
	 * specified dispatch.
	 *
	 * @param dispatch - the dispatch a method is required for.
	 * @param form - the ActionForm for the current request.
	 * @return the Method to delegate to for the specified the dispatch.
	 */
	private Method getExecuteMethod(String dispatch, ActionForm form) throws Exception {

		// Determine the method name based on the dispatch
		String methodName = null;
		if ((dispatch==null)||(dispatch.trim().equals(""))) {
			// No dispatch provided, use the default execute method
			methodName = "executeDefault";
		} else {
			// Convert the dispatch to a method name
			methodName = "execute" + Character.
				toUpperCase(dispatch.charAt(0)) + dispatch.substring(1);
		}
		logger.debug("locating method with name '" + methodName + "'");

		// Locate a suitable method that has this name.
		// Note: use getClass() to refer to the subclass class
		Method method = (Method) executeMethods.get(dispatch);
		if (method==null) {
			// A Method for this dispatch has not been located yet
			try {
				// Try executeXXXX(Request, form.getClass());
				method = getClass().getMethod(methodName,
					new Class[] {Request.class, form.getClass()});
			} catch (Exception e) {
				logger.debug("could not find " + methodName +
					"(Request, form.getClass()) method: " + e.getMessage());
				try {
					// Try executeXXXX(Request, ActionForm);
					method = getClass().getMethod(methodName,
						new Class[] {Request.class, ActionForm.class});
				} catch (Exception e2) {
					logger.debug("could not find " + methodName +
						"(Request, ActionForm) method: " + e2.getMessage());
					try {
						// Try executeXXXX(Request);
						method = getClass().getMethod(methodName,
							new Class[] {Request.class});
					} catch (Exception e3) {
						logger.debug("could not find " + methodName +
							"(Request) method: " + e3.getMessage());
					}
				}
			}
			// Cache the method (if any) under the dispatch
			if (method!=null) {
				executeMethods.put(dispatch, method);
			}
		}

		if (method==null) {
			// Could not locate a method for the specified dispatch
			logger.error("could not find execute method for dispatch='"+dispatch+"'");
			throw new Exception("Unhandled dispatch '" + dispatch + "'");
		} else {
			// And return it
			logger.debug("found method=" + method);
			return method;
		}

	}

	/**
	 * Called before the dispatch is executed.
	 *
	 * @param request - the current Request.
	 */
	public void preExecute(Request request) throws Exception {
		// Default implementation does nothing
	}

	/**
	 * The method that is executed when no dispath is provided.
	 *
	 * @param request - the current Request.
	 * @return
	 */
	public ActionForward executeDefault(Request request) {
		// Default implementation simply gos to the success forward
		return (request.getMapping().findForward("success"));
	}

	/**
	 * Called after the dispatch has been executed.
	 *
	 * @param request - the current Request.
	 */
	public void postExecute(Request request) throws Exception {
		// Default implementation does nothing
	}

	/**
	 * Adds the specified error to any existing errors for the request.
	 *
	 * @param request - the current Request.
	 * @param property - the property name.
	 * @param error
	 */
	protected void addError(Request request, String property, ActionMessage error) {
		ActionErrors existingErrors = (ActionErrors)
				request.getRequest().getAttribute(Globals.ERROR_KEY);
		if (existingErrors==null) {
			existingErrors = new ActionErrors();
			request.getRequest().setAttribute(Globals.ERROR_KEY, existingErrors);
		}
		existingErrors.add(property, error);
	}

	/**
	 * Adds the specified error to any existing errors for the request.
	 *
	 * @param request - the current Request.
	 * @param errorKey - identifying an error message in the resources.
	 */
	protected void addGlobalError(Request request, String errorKey) {
		ActionMessage error = new ActionMessage(errorKey);
		addError(request, ActionMessages.GLOBAL_MESSAGE, error);
	}

	/**
	 * Adds the specified error to any existing errors for the request.
	 *
	 * @param request - the current Request.
	 * @param errorKey - identifying an error message in the resources.
	 * @param arg0 - used to format the message.
	 */
	protected void addGlobalError(Request request, String errorKey, Object arg0) {
		ActionMessage error = new ActionMessage(errorKey, arg0);
		addError(request, ActionMessages.GLOBAL_MESSAGE, error);
	}

	/**
	 * Adds the specified error to any existing errors for the request.
	 *
	 * @param request - the current Request.
	 * @param errorKey - identifying an error message in the resources.
	 * @param args - used to format the message.
	 */
	protected void addGlobalError(Request request, String errorKey, Object[] args) {
		ActionMessage error = new ActionMessage(errorKey, args);
		addError(request, ActionMessages.GLOBAL_MESSAGE, error);
	}

	/**
	 * Adds the specified message to any existing messages for the request.
	 *
	 * @param request - the current Request.
	 * @param property - the form property the message is associated with.
	 * @param message - the ActionMessage to add.
	 */
	protected void addMessage(Request request, String property, ActionMessage message) {
		ActionMessages existingMessages = (ActionMessages)
				request.getRequest().getAttribute(Globals.MESSAGE_KEY);
		if (existingMessages==null) {
			existingMessages = new ActionMessages();
			request.getRequest().setAttribute(Globals.MESSAGE_KEY, existingMessages);
		}
		existingMessages.add(property, message);
	}

	/**
	 * Adds the specified message to any existing messages for the request.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 */
	protected void addGlobalMessage(Request request, String messageKey) {
		ActionMessage message = new ActionMessage(messageKey);
		addMessage(request, ActionMessages.GLOBAL_MESSAGE, message);
	}

	/**
	 * Adds the specified message to any existing messages for the request.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 * @param arg0 - used to format the message.
	 */
	protected void addGlobalMessage(Request request, String messageKey, Object arg0) {
		ActionMessage message = new ActionMessage(messageKey, arg0);
		addMessage(request, ActionMessages.GLOBAL_MESSAGE, message);
	}

	/**
	 * Adds the specified message to any existing messages for the request.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 * @param args - used to format the message.
	 */
	protected void addGlobalMessage(Request request, String messageKey, Object[] args) {
		ActionMessage message = new ActionMessage(messageKey, args);
		addMessage(request, ActionMessages.GLOBAL_MESSAGE, message);
	}

	/**
	 * Returns the a message for the specified key and args.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 * @param args - used to format the message.
	 * @return the messages
	 */
	protected String getMessage(Request request, String messageKey, Object[] args) {
		// Get the MessageResources associated with the current request
		MessageResources resources = getResources(request.getRequest());
		// Get the Locale for the current request
		Locale locale = getLocale(request.getRequest());
		// Get the message
		if (args == null) {
			return (resources.getMessage(locale, messageKey));
		} else {
			return (resources.getMessage(locale, messageKey, args));
		}
	}

	/**
	 * Returns the a message for the specified key.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 */
	public String getMessage(Request request, String messageKey) {
		return getMessage(request, messageKey, null);
	}

	/**
	 * Returns the a message for the specified key and arg.
	 *
	 * @param request - the current Request.
	 * @param messageKey - identifying a message in the resources.
	 * @param arg0 - used to format the message.
	 */
	public String getMessage(Request request, String messageKey, Object arg0) {
		return getMessage(request, messageKey, new Object[] {arg0});
	}

	/**
	 * Overrides the default implementation to warn if any messages are being
	 * overriden by the save.
	 *
	 * @param request - the current Request.
	 * @param messages - the ActionMessages to save.
	 */
	protected void saveMessages(HttpServletRequest request, ActionMessages messages) {

		// Warn if overriding existing messages
		if (request.getAttribute(Globals.MESSAGE_KEY)!=null) {
			logger.warn("overriding existing messages=" + request.getAttribute(Globals.MESSAGE_KEY));
		}
		// Continue regular processing
		super.saveMessages(request, messages);

	}

	/**
	 * Override the default implementation to warn if any errors are being
	 * overriden by the save.
	 *
	 * @param request - the current Request.
	 * @param errors - the ActionMessages to save.
	 */
	protected void saveErrors(HttpServletRequest request, ActionMessages errors) {

		// Warn if overriding existing errors
		if (request.getAttribute(Globals.ERROR_KEY)!=null) {
			logger.warn("overriding existing errors=" + request.getAttribute(Globals.ERROR_KEY));
		}
		// Continue regular processing
		super.saveErrors(request, errors);

	}

	/**
	 * Uses copy Utils to copy the source object to the destination object
	 * using the resources for the current module and the locale for the
	 * users request.
	 *
	 * @param request - the current Request.
	 * @param dest - the object to copy values to.
	 * @param source - the object to copy values from.
	 * @exception CopyException - if there is a problem copying.
	 */
	protected void copy(Request request, Object dest, Object source) throws CopyException {
		// Get the Locale for the current request
		Locale locale = getLocale(request.getRequest());
		// Get the ConvertUtils to use for the copy
		ConvertUtils cu = getConvertUtils(request.getRequest());
		// Copy from the source to the destination object
		CopyUtils.copy(dest, source, cu, locale);
	}

	/**
	 * Return the default ConvertUtils for the current module.
	 *
	 * @param request - the current HttpServletRequest.
	 */
	protected ConvertUtils getConvertUtils(HttpServletRequest request) {
		if (convertUtils==null) {
			// Get the MessageResources for the current module.
			MessageResources resources = getResources(request);
			// Create the ConvertUtils
			convertUtils = ConvertUtils.createDefaultInstance(resources);
		}
		return convertUtils;
	}

}
