<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=ApplicationException.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=util -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.util;

/**
 * Super class of all Exceptions thrown by the application. ApplicationException
 * has an error key that identifies the error and can be used to lookup a user
 * friendly error message. It also can store additional information related to
 * the exception.
 */
public class ApplicationException extends Exception {

	/**
	 * Additional data related to the error that can be used to render a user
	 * friendly error message.
	 */
	private Object[] args = null;

	/**
	 * The root cause of this exception
	 */
	private Throwable cause = null;

	/**
	 * Construct a new ApplicationException
	 *
	 * @param key - identifying the exception type.
	 * @param args - additional information about the exception.
	 * @param cause - the root cause of the exception.
	 */
	public ApplicationException(String key, Object[] args, Throwable cause) {
		super(key);
		this.args = args;
		this.cause = cause;
	}

	/**
	 * Construct a new ApplicationException
	 *
	 * @param key - identifying the exception type.
	 * @param arg - additional information about the exception.
	 * @param cause - the root cause of the exception.
	 */
	public ApplicationException(String key, Object arg, Throwable cause) {
		this(key, new Object[] {arg}, cause);
	}

	/**
	 * Return the extra information associated with this.
	 *
	 * @return the array of extra information.
	 */
	public Object[] getArgs() {
		return args;
	}

	/**
	 * Returns the error key for this.
	 *
	 * @return the error key.
	 */
	public String getKey() {
		return getMessage();
	}

	/**
	 * Returns the cause of this.
	 *
	 * @return the cause.
	 */
	public Throwable getCause() {
		return cause;
	}

}
