<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=CalendarTag.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=web.taglib -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.web.taglib;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import org.apache.struts.taglib.TagUtils;
import org.apache.struts.taglib.html.LinkTag;
import org.apache.struts.taglib.nested.NestedNameSupport;
import org.apache.struts.taglib.nested.NestedPropertyHelper;

/**
 * Writes out required html to make use of the javascript Calendar Popup
 * (http://www.mattkruse.com/javascript/calendarpopup/).
 *
 * This tag outputs html that depends on the getDocumentFormElement() method
 * from common.js to locate the correct form element.
 */
public class CalendarTag extends LinkTag implements NestedNameSupport {

	/**
	 * Request key used to store the number of CalendarTags rendered for the
	 * current request. Used to create unique IDs for html elements.
	 */
	private static final String REQUEST_KEY_COUNT = "CalendarTag.count";

	/**
	 * The original name before nesting
	 */
	private String originalName = null;

	/**
	 * The original property before nesting
	 */
	private String originalProperty = null;

	/**
	 * The originally specified id
	 */
	private String originalId = null;

	/**
	 * The originally specified onclick handler
	 */
	private String originalOnclick = null;

	/**
	 * The contents of the tag body
	 */
	private String body = null;

	/**
	 * The number count of this calendar tag.
	 */
	private int count = -1;

	/**
	 * Prepare and output out the start tag for the calendar.
	 *
	 * @return int JSP continuation directive.
	 * @exception JspException if the tag could not be prepared
	 */
	public int doStartTag() throws JspException {

		// Preserve the original properties
		originalName = getName();
		originalProperty = getProperty();
		originalOnclick = getOnclick();
		originalId = getId();

		// Set the nested properties
		HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
		NestedPropertyHelper.setNestedProperties(request, this);

		// Get the next count
		count = getNextCount();

		// Setup the id (if it has not already been set)
		setupId();

		// Setup the onclick
		setupOnclick();

		// Build up the output
		StringBuffer buf = new StringBuffer();

		// Append the javscript
		prepareJavascript(buf);

		// Append the anchor
		prepareAnchor(buf);

		// Write the results out
		TagUtils.getInstance().write(pageContext, buf.toString());

		return (EVAL_BODY_BUFFERED);

	}

	/**
	 * Only evaluate the body once.
	 */
	public int doAfterBody() throws JspException {

		// Save any body content for later
		if (bodyContent!=null) {
			body = bodyContent.getString();
		}
		return (SKIP_BODY);
	}

	/**
	 * Write out the body content and the calandar end tag.
	 *
	 * @return EVAL_PAGE
	 * @exception JspException if there is a problem writing the output
	 */
	public int doEndTag() throws JspException {

		StringBuffer buf = new StringBuffer();

		// If there was no body content, use a default value
		if (body==null) {
			body = "Select";
		}

		// Append the body
		buf.append(body);
		// Reset the body in case the tag is used again
		body = null;

		// Output the end tag
		buf.append("</a>\n");

		// Write the buffer to the page
		TagUtils.getInstance().write(pageContext, buf.toString());

		// Reset the properties
		setName(originalName);
		setProperty(originalProperty);
		setId(originalId);
		setOnclick(originalOnclick);

		// Continue evaluting page
		return (EVAL_PAGE);

	}

	/**
	 * Release values
	 */
	public void release() {

		super.release();
		// Release property values
		originalName = null;
		originalProperty = null;
		originalId = null;
		originalOnclick = null;
		body = null;

	}

	// --------------- Start Private Methods ------------------------------ //

	/**
	 * Sets the id if it has not been explicitely set.
	 */
	private void setupId() {
		if ((getId()==null)||(getId().equals(""))) {
			// Id has not been explicitely set, set a unique id
			setId("anchor"+count);
		}
	}

	/**
	 * Sets up the onclick handler so that the hidden field is set when the
	 * checkbox is clicked.
	 */
	private void setupOnclick() throws JspException {

		// Get the date mask to use
		String dateMask = TagUtils.getInstance().message(
				pageContext, null, null, "calendarTag.dateMask", null);
		if (dateMask==null) {
			throw new JspException(messages.getMessage("calendarTag.dateMaskMissing"));
		}

		StringBuffer buf = new StringBuffer();
		// Append the original onclick
		if (getOnclick()!=null) {
			buf.append(originalOnclick).append("; ");
		}
		// Append the calendar onclick handler
		buf.append("cal").append(count).append(".select(getDocumentFormElement('");
		if (indexed) {
			prepareIndex(buf, name);
		}
		buf.append(property).append("'),'").append(getId());
		buf.append("','").append(dateMask).append("'); return false;");

		// Set the new onclick
		setOnclick(buf.toString());

	}

	/**
	 * Append the anchor tag html to the specified StringBuffer.
	 *
	 * @param buf - the StringBuffer to write html to.
	 */
	private void prepareAnchor(StringBuffer buf) throws JspException {

		// Start tag, and name and id
		buf.append("\n<a href=\"#\"");
		buf.append(" name=\"").append(getId()).append("\"");
		buf.append(" id=\"").append(getId()).append("\"");

		// Access key
		if (accesskey != null) {
			buf.append(" accesskey=\"");
			buf.append(accesskey);
			buf.append("\"");
		}

		// Tab index
		if (tabindex != null) {
			buf.append(" tabindex=\"");
			buf.append(tabindex);
			buf.append("\"");
		}

		// Prepate the styles
		buf.append(prepareStyles());

		// Prepare the event handlers
		buf.append(prepareEventHandlers());

		// Close the anchor tag
		buf.append(getElementClose());

	}

	/**
	 * Appends the javascript used to initialise the CalendarPopup.
	 *
	 * @param buf - the StringBuffer to append to.
	 */
	private void prepareJavascript(StringBuffer buf) {
		buf.append("\n<script language=\"JavaScript\">\n");
		buf.append("\nvar cal").append(count).append(" = new CalendarPopup();\n");
		buf.append("\n</script>\n");
	}

	/**
	 * Returns the next count for the current request.
	 */
	private int getNextCount() {

		// Get the existing count (if any)
		Integer count = (Integer) pageContext.getRequest().getAttribute(REQUEST_KEY_COUNT);
		if (count == null) {
			// First usage of the tag for this request
			pageContext.getRequest().setAttribute(REQUEST_KEY_COUNT, new Integer(1));
			return 1;
		} else {
			// Count already exists, increment and return.
			pageContext.getRequest().setAttribute(REQUEST_KEY_COUNT, new Integer(count.intValue() + 1));
			return (count.intValue() + 1);
		}

	}

	// --------------- End Private Methods -------------------------------- //

}
