<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=CheckboxTag.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=web.taglib -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.web.taglib;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import org.apache.struts.taglib.TagUtils;
import org.apache.struts.taglib.nested.NestedNameSupport;
import org.apache.struts.taglib.nested.NestedPropertyHelper;

/**
 * CheckboxTag renders boolean properties using a checkbox and stores the value
 * in a hidden field so that a cgi value is always posted when the form is
 * submitted. This is a workaround for the limitation of html checkbox inputs
 * where the value is only includes when the checkbox is selected.
 *
 * This tag outputs html that depends on the getDocumentFormElement() method
 * from common.js to locate the correct form element.
 */
public class CheckboxTag extends org.apache.struts.taglib.html.CheckboxTag implements NestedNameSupport {

	/**
	 * The original name before nesting was applied
	 */
	private String originalName = null;

	/**
	 * The original property before nesting was applied
	 */
	private String originalProperty = null;

	/**
	 * The original onclick
	 */
	private String originalOnclick = null;

	/**
	 * Generate the required input tag.
	 *
	 * @exception JspException if a JSP exception has occurred
	 */
	public int doStartTag() throws JspException {

		// Store the original properties so we can change them back later
		originalName = getName();
		originalProperty = getProperty();
		originalOnclick = getOnclick();

		// Setup the nested name and property
		HttpServletRequest request = (HttpServletRequest)pageContext.getRequest();
		NestedPropertyHelper.setNestedProperties(request, this);

		// Setup the onclick
		setupOnclick();

		// Build up the output
		StringBuffer buf = new StringBuffer();

		// Prepare the checkbox html
		prepareCheckbox(buf);

		// Prepare the hidden html field (used to store actual property value)
		prepareHiddenField(buf);

		// Prepare the javascript method (used to set the value of hidden field)
		prepareJavascript(buf);

		// Write the results out
		TagUtils.getInstance().write(pageContext, buf.toString());

		// Skip the body
		return (SKIP_BODY);

	}

	/**
	 * Reset original properties and continue evaluating page
	 *
	 * @return EVAL_PAGE
	 */
	public int doEndTag() {

		// Reset the properties
		setName(originalName);
		setProperty(originalProperty);
		setOnclick(originalOnclick);

		// Continue evaluting page
		return (EVAL_PAGE);

	}

	/**
	 * Release values
	 */
	public void release() {

		super.release();
		// Release property values
		originalName = null;
		originalProperty = null;
		originalOnclick = null;

	}

	// --------------- Start Private Methods ------------------------------ //

	/**
	 * Sets up the onclick handler so that the hidden field is set when the
	 * checkbox is clicked.
	 */
	private void setupOnclick() {
		if (getOnclick()!= null) {
			// An onclick has already been defined, add to the end
			setOnclick(getOnclick() + "; " + getJavascriptMethodName() + "()");
		} else {
			setOnclick(getJavascriptMethodName() + "()");
		}
	}

	/**
	 * Append the checkbox tag html to the specified StringBuffer.
	 *
	 * @param buf - the StringBuffer to write html to.
	 */
	private void prepareCheckbox(StringBuffer buf) throws JspException {

		// The start
		buf.append("\n<input type=\"checkbox\"");

		// The name. Note: preceed the property name with and "_" because it is
		// not actually used to populate the form. Instead we will use the
		// value from the hidden field to populate the form.
		buf.append(" name=\"").append("_");
		if (indexed) {
			prepareIndex(buf, name);
		}
		buf.append(property).append("\"");

		// Access key
		if (accesskey != null) {
			buf.append(" accesskey=\"");
			buf.append(accesskey);
			buf.append("\"");
		}

		// Tab index
		if (tabindex != null) {
			buf.append(" tabindex=\"");
			buf.append(tabindex);
			buf.append("\"");
		}

		// Append "checked" if necessary
		if (isChecked()) {
			buf.append(" checked=\"checked\"");
		}

		// Prepare event handlers
		buf.append(prepareEventHandlers());

		// Prepare styles
		buf.append(prepareStyles());

		// Close the checkbox tag
		buf.append(getElementClose());

	}

	/**
	 * Append the hidden field html to the specified StringBuffer.
	 *
	 * @param buf - the StringBuffer to write html to.
	 */
	private void prepareHiddenField(StringBuffer buf) throws JspException {
		buf.append("\n<input type=\"hidden\" name=\"").append(property);
		buf.append("\" value=\"").append(isChecked()).append("\"");
		buf.append(getElementClose());
	}

	/**
	 * Append the javascript that sets the value of the hidden field based on
	 * whether the checkbox is selected to the specified StringBuffer.
	 *
	 * @param buf - the StringBuffer to write the javascript to.
	 */
	private void prepareJavascript(StringBuffer buf) {

		buf.append("\n<script language=\"JavaScript\">");
		buf.append("\n  function " + getJavascriptMethodName() + "() {");
		buf.append("\n    var checkbox = getDocumentFormElement(\"_" + this.property + "\");");
		buf.append("\n    var hidden = getDocumentFormElement(\"" + this.property + "\");");
		buf.append("\n    if (checkbox.checked ) {");
		buf.append("\n      hidden.value='true';");
		buf.append("\n    } else {");
		buf.append("\n      hidden.value='false';");
		buf.append("\n    }");
		buf.append("\n  }");
		buf.append("\n</script>");

	}

	/**
	 * Returns the name of the javascript method that sets the value of the
	 * hidden field based on whether the checkbox is selected. The name is
	 * based on the property name and therefore should be unique on the page.
	 *
	 * @return
	 */
	private String getJavascriptMethodName() {

		StringBuffer buf = new StringBuffer("update");
		for (int i = 0; i < property.length(); i++) {
			char c = property.charAt(i);
			// Only use letters and digits to prevent javascript errors
			if (Character.isLetterOrDigit(c)) {
				buf.append(c);
			}
		}
		return buf.toString();

	}

	// --------------- End Private Methods -------------------------------- //

}
