<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=ExceptionHandler.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=util.action -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.util.action;

import java.util.ArrayList;
import java.util.Arrays;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.config.ExceptionConfig;
import org.apache.struts.util.MessageResources;

import ${project.rootPackage}.util.ApplicationException;

/**
 * Extension to the struts ExceptionHandler to handle ApplicationExceptions.
 */
public class ExceptionHandler extends org.apache.struts.action.ExceptionHandler {

	/**
	 * Extends the default functionality.
	 */
	public ActionForward execute(
		Exception ex,
		ExceptionConfig ac,
		ActionMapping mapping,
		ActionForm formInstance,
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException {

		// Execute the base functionality
		ActionForward forward = super.execute(ex, ac, mapping, formInstance, request, response);

		// Additional processing for ApplicationExceptions
		if (ex instanceof ApplicationException) {
			// Get the exception message
			String message = getExceptionMessage(request, ex);
			// Construct an ActionMessage ExceptionConfig key and the message
			ActionMessage error = new ActionMessage(ac.getKey(), message);
			// Store the exception (overwrites what super.execute() did)
			request.setAttribute(Globals.EXCEPTION_KEY, ex);
			this.storeException(request, ac.getKey(), error, forward, ac.getScope());
		}

		// Return the forward
		return forward;

	}

	/**
	 * Returns the message to use for the specified Throwable. This method
	 * handles the extra parameters present in ApplicationExceptions.
	 *
	 * @param request - the current request.
	 * @param e - the Throwable to get a message for.
	 */
	protected String getExceptionMessage(HttpServletRequest request, Throwable e) {

		if (e instanceof ApplicationException) {

			// Construct the localised message
			ApplicationException ae = (ApplicationException) e;
			// Get the message key
			String key = ae.getKey();
			// Contsruct the list of args using supplied args and the cause
			// message as the last argument (if there is a cause).
			ArrayList args = new ArrayList();
			if (ae.getArgs()!=null) {
				args.addAll(Arrays.asList(ae.getArgs()));
			}
			if (ae.getCause()!=null) {
				args.add(getExceptionMessage(request, ae.getCause()));
			}
			// Now format and return the message
			return getResources(request).getMessage(key, args.toArray());

		} else {

			// Not an ApplicationException. Return the localised message.
			return e.getLocalizedMessage();

		}

	}

	/**
	 * Returns the default MessageResources for the current module.
	 *
	 * @param request - the current request.
	 */
	protected MessageResources getResources(HttpServletRequest request) {
		return ((MessageResources) request.getAttribute(Globals.MESSAGES_KEY));
	}

}
