<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=FieldChecks.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=util -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.util;

import java.io.Serializable;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.validator.Field;
import org.apache.log4j.Logger;

/**
 * Additional validations for Jakarta Commons Validator.
 */
public class FieldChecks implements Serializable {

	/**
	 * Log4j Logger for this class
	 */
	public static final Logger logger = Logger.getLogger(FieldChecks.class);

	/**
	 * <p>Short circuit validator for nested, non indexed fields with null
	 * parents.
	 *
	 * <p>The purpose of this validator is to prevent other validators from
	 * executing on a nested property whose parent is null. No validation
	 * errors are added but the validation is discontinued for that field.
	 * This is usefull for dynamic forms where there presence of certain
	 * objects on the form is not guaranteed.
	 *
	 * @param bean - the object being validated
	 * @param field - the field oon the object being validated
	 * @return true if the fields parent is present
	 */
	public static boolean validateIfParentPresent(Object bean, Field field) {

		// If bean is null, then parent is null
		if (bean==null) {
			return false;
		}

		// Determine the parent property is non null by recursing through
		// the non nested properties and verifying they return something.
		StringTokenizer st = new StringTokenizer(field.getProperty(),".");
		String property = null;
		while (st.hasMoreTokens()) {
			property = st.nextToken();
			if (!st.hasMoreTokens()) {
				// This is the last property, meaning the parent is present.
				return true;
			}
			try {
				bean = PropertyUtils.getProperty(bean, property);
			} catch (Exception e) {
				logger.error("exception getting property value", e);
				return false;
			}
			if (bean==null) {
				// The parent (or grandparent) object not present.
				return false;
			}
		}

		// The parent property is present.
		return true;

	}

}
