<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=IdentityUtils.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=util.identity -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.util.identity;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Provides utility methods for Identifiables.
 */
public class IdentityUtils {

	/**
	 * Removes the first Identifiable in the supplied Collection that has the
	 * specified id.
	 *
	 * @param id - the id of the Identifiable to remove.
	 * @param identifiables - the Collection to remove an Identifiable from.
	 * @return true if an Identifiable was actually removed from the Collection.
	 */
	public static boolean removeIdentifiable(Long id, Collection identifiables) {

		if ((identifiables!=null) && (!identifiables.isEmpty())) {
			Identifiable identifiable = null;
			for (Iterator iter = identifiables.iterator(); iter.hasNext(); ) {
				identifiable = (Identifiable) iter.next();
				if ((identifiable!=null) &&
						(identifiable.getId()==null ? id==null : identifiable.getId().equals(id))) {
					// Found an Identifiable with the specified id.
					iter.remove();
					return true;
				}
			}

		}
		// Did not remove it
		return false;

	}

	/**
	 * Returns a negative id that is unique within the specified Identifiables.
	 *
	 * @param identifiables - collection of Identifiable objects.
	 * @return a unique negative id.
	 */
	public static Long getUniqueNegativeId(Collection identifiables) {

		long uniqueId = -1;
		if ((identifiables!=null) && (!identifiables.isEmpty())) {
			// Populate a Set with all used IDs
			Set usedIds = new HashSet(identifiables.size());
			Identifiable identifiable = null;
			for (Iterator iter = identifiables.iterator(); iter.hasNext(); ) {
				identifiable = (Identifiable) iter.next();
				if ((identifiable!=null) && (identifiable.getId()!=null)) {
					// Add this id to the set of used IDs
					usedIds.add(identifiable.getId());
				}
			}
			// Determine a unique id using the used IDs
			while (usedIds.contains(new Long(uniqueId))) {
				uniqueId--;
			}
		}
		return new Long(uniqueId);

	}

}
