<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=PagedData.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=util.dao -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.util.dao;

import java.util.ArrayList;

/**
 * Encapsulates a single page of data.
 */
public class PagedData extends ArrayList {

	/**
	 * The page number for this
	 */
	private int pageNumber;

	/**
	 * The maximum number of results for a page
	 */
	private int pageSize;

	/**
	 * The total number of records
	 */
	private int totalRecords;

	/**
	 * Constructs an empty PageList with the specified page number and size.
	 *
	 * @param pageNumber - the page number (at least one)
	 * @param pageSize - the maximum number of results for this page (needs
	 * to be at least one).
	 */
	public PagedData(int pageNumber, int pageSize) {
		super(pageSize);
		if (pageNumber <= 0) {
			throw new IllegalArgumentException("Illegal pageNumber=" + pageNumber);
		}
		if (pageSize <= 0) {
			throw new IllegalArgumentException("Illegal pageSize=" + pageSize);
		}
		this.pageNumber = pageNumber;
		this.pageSize = pageSize;
	}

	/**
	 * Sets the total number of records across all pages.
	 *
	 * @param totalRecords
	 */
	public void setTotalRecords(int totalRecords) {
		this.totalRecords = totalRecords;
	}

	/**
	 * Returns the page number for this
	 *
	 * @return
	 */
	public int getPageNumber() {
		return pageNumber;
	}

	/**
	 * Returns the total number of pages.
	 *
	 * @return
	 */
	public int getTotalPages() {
		return (totalRecords-1)/pageSize + 1;
	}

	/**
	 * Returns true if this is the last page.
	 *
	 * @return
	 */
	public boolean isLastPage() {
		return (getPageNumber()>=getTotalPages());
	}

	/**
	 * Returns the next page number.
	 *
	 * @return
	 */
	public int getNextPageNumber() {
		return (getPageNumber()+1);
	}

	/**
	 * Returns the previous page number.
	 *
	 * @return
	 */
	public int getPreviousPageNumber() {
		return (getPageNumber()-1);
	}

	/**
	 * Returns an acending (1,2..) list of Integers of all pages up to but not
	 * including the current page.
	 *
	 * @return
	 */
	public ArrayList getPreviousPages() {
		ArrayList list = new ArrayList(pageNumber-1);
		for (int i = 1; i < pageNumber; i++) {
			list.add(new Integer(i));
		}
		return list;
	}

	/**
	 * Returns an acending (1,2..) list of Integers of all pages after the current page.
	 *
	 * @return
	 */
	public ArrayList getNextPages() {
		int totalPages = getTotalPages();
		ArrayList list = new ArrayList(totalPages);
		for (int i = pageNumber+1; i <= totalPages; i++) {
			list.add(new Integer(i));
		}
		return list;
	}

	/**
	 * toString() impl
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("[").append(super.toString()).append("{");
		buf.append("pageNumber=").append(pageNumber);
		buf.append(", pageSize=").append(pageSize);
		buf.append(", totalRecords=").append(totalRecords);
		buf.append("}]");
		return buf.toString();
	}

}
