<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=application -->
<#-- version=1.0 -->
<#-- fileNameTemplate=build.xml -->
<#-- relativeOutputDirectory=ant/generic -->
<#-- END TEMPLATE HEADER -->
<?xml version="1.0" encoding="ISO-8859-1" ?>

<!--

  Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.

  Modification and redistribution of this file is permitted subject to the
  terms and conditions of the RapidJ End User License Agreement:
  http://www.codecanvas.com.au/rapidj/license.php

  {{$RapidJ$}}

-->

<!--

${f.formatLines(project.copyright, '  ', 80)}

-->

<project name="${project.name?xml}" default="compile" basedir=".">
<#noparse>

<!-- =============== Customisable Build Property Definitions ============== -->

<!--

  The following properties can be customised either here or by defining them
  in the build.properties file.

  NOTE: Properties are immutable. Once they have been set for the first time
  using the <property> task they cannot be changed.

  app.name             The name of the application being built. Is used when
                       creating directories and files related to the build
                       process. Defaults to "myapp".

  app.version          The version of the application being built.

  build.dir.name       The name of the directory under the project directory
                       used to construct the exploded war file. Defaults to
                       "build".

  dist.dir.name        The name of the directory under the project directory
                       used to store the application war file. Defaults to
                       "dist".

  javadoc.dir.name     The name of the directory under the project directory
                       used to store the javadoc generated by the javadoc
                       target. Defaults to "javadoc".

  deploy.dir           The directory where the deploy target will copy the
                       application war file. This can be set to the deploy
                       directory of your application server to easily deploy
                       the application. Also used by the remove target.

  j2ee.jar             The location of the j2ee.jar which is required to
                       compile the Java source files.


   {{$RapidJ$}}

-->

<!-- Load properties from build.properties -->
  <property file="build.properties"/>

<!-- Set default values for applicable unset properties -->
  <property name="app.name"            value="myapp"/>
  <property name="app.version"         value="1"/>
  <property name="build.dir.name"      value="build"/>
  <property name="dist.dir.name"       value="dist"/>
  <property name="javadoc.dir.name"    value="javadoc"/>


<!-- ===================== Build Property Definitions ========================= -->

<!--

  The following properties cannot be customised without making changes to the
  rest of the script.

-->

  <property name="project.dir"     value="${basedir}/../../"/>
  <property name="build.dir"       value="${project.dir}/${build.dir.name}"/>
  <property name="dist.dir"        value="${project.dir}/${dist.dir.name}"/>
  <property name="javadoc.dir"     value="${project.dir}/${javadoc.dir.name}"/>
  <property name="src.dir"         value="${project.dir}/src"/>
  <property name="web.dir"         value="${project.dir}/web"/>


<!--  ==================== Compilation Control Options ==================== -->

<!--

  Property definitions for options passed to javac in the "compile" target.

  compile.debug        Should compilation include the debug option?

  compile.deprecation  Should compilation include the deprecation option?

  compile.optimize     Should compilation include the optimize option?

-->

  <property name="compile.debug"         value="true"/>
  <property name="compile.deprecation"   value="false"/>
  <property name="compile.optimize"      value="false"/>


<!-- ==================== External Dependencies =========================== -->

<!--

  Property definitions that define the location of external JAR files on which
  the application depends. Generally these JAR files are used for two purposes.
  1) To compile the application source files, and
  2) To be included in the application WAR file in the WEB-INF/lib directory.

-->

  <property name="struts.jar"             value="../../lib/struts.jar"/>
  <property name="commons-beanutils.jar"  value="../../lib/commons-beanutils.jar"/>
  <property name="commons-digester.jar"   value="../../lib/commons-digester.jar"/>
  <property name="commons-logging.jar"    value="../../lib/commons-logging.jar"/>
  <property name="commons-validator.jar"  value="../../lib/commons-validator-1.1.4.jar"/>
  <property name="log4j.jar"              value="../../lib/log4j-1.2.9.jar"/>
  <property name="freemarker.jar"         value="../../lib/freemarker.jar"/>
  <property name="jakarta-oro.jar"        value="../../lib/jakarta-oro-2.0.8.jar"/>


<!-- ==================== Compilation Classpath =========================== -->

<!--

  Create the classpath that is required to compile the application.

-->

  <path id="compile.classpath">

    <!-- Include all JAR files required to compile the application -->
    <pathelement location="${j2ee.jar}"/>
    <pathelement location="${struts.jar}"/>
    <pathelement location="${commons-beanutils.jar}"/>
    <pathelement location="${commons-digester.jar}"/>
    <pathelement location="${commons-logging.jar}"/>
    <pathelement location="${commons-validator.jar}"/>
    <pathelement location="${log4j.jar}"/>
    <pathelement location="${freemarker.jar}"/>

  </path>


<!-- ==================== Clean Target ==================================== -->

<!--

  The "clean" target deletes the "build" and "dist" directories. This will
  ensure that the application will be built from scratch next time.

-->

  <target name="clean"
      description="Delete old build and dist directories">
    <delete dir="${build.dir}"/>
    <delete dir="${dist.dir}"/>
  </target>


<!-- ==================== Prepare Target ================================== -->

<!--

  The "prepare" target is used to create the main build related directories and
  copy in any static content required by the web application.

-->

  <target name="prepare"
      description="Create build directories and copy static content">

    <!-- Create build directories as needed -->
    <mkdir dir="${build.dir}"/>
    <mkdir dir="${build.dir}/WEB-INF"/>
    <mkdir dir="${build.dir}/META-INF"/>
    <mkdir dir="${build.dir}/WEB-INF/classes"/>

    <!-- Setup the filter used when copying files -->
    <filter filtersfile="build.properties"/>

    <!-- Copy the web application static content -->
    <copy todir="${build.dir}">
      <fileset dir="${web.dir}" excludes="WEB-INF/web.xml,**/*.shd"/>
    </copy>

    <!-- Copy (with filtering) web.xml -->
    <copy    file="${web.dir}/WEB-INF/web.xml"
            todir="${build.dir}/WEB-INF"
        filtering="true"
        overwrite="true"/>

    <!-- Copy application resources -->
    <copy  todir="${build.dir}/WEB-INF/classes">
      <fileset dir="${src.dir}"
          excludes="application.properties,**/*.java,**/*.shd"/>
    </copy>

    <!-- Copy (with filtering) application.properties -->
    <copy   file="${src.dir}/application.properties"
           todir="${build.dir}/WEB-INF/classes/"
        filtering="true"
        overwrite="true"/>

    <!-- Copy external dependencies as required -->
    <mkdir  dir="${build.dir}/WEB-INF/lib"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${struts.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-beanutils.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-digester.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-logging.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${commons-validator.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${log4j.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${freemarker.jar}"/>
    <copy todir="${build.dir}/WEB-INF/lib" file="${jakarta-oro.jar}"/>

  </target>


<!-- ==================== Compile Target ================================== -->

<!--

  The "compile" target uses the javac task to compile Java source files in
  ${src.dir} to the WEB-INF/classes directory.

-->

  <target name="compile" depends="prepare"
      description="Compile Java sources">

    <!-- Make sure the location of j2ee.jar has been set -->
    <fail unless="j2ee.jar" message="j2ee.jar not set"/>

    <!-- Compile Java classes as necessary -->
    <javac srcdir="${src.dir}"
          destdir="${build.dir}/WEB-INF/classes"
            debug="${compile.debug}"
      deprecation="${compile.deprecation}"
         optimize="${compile.optimize}">
        <classpath refid="compile.classpath"/>
    </javac>

  </target>


<!-- ==================== Dist Target ===================================== -->

<!--

  The "dist" target creates a WAR file containing the application.

-->

  <target name="dist" depends="compile"
      description="Create application WAR file">

	<!-- Create the dist directory as needed -->
	<mkdir dir="${dist.dir}"/>

    <!-- Increment the build number -->
    <buildnumber/>

    <!-- Create application WAR file -->
    <jar jarfile="${dist.dir}/${app.name}.war" basedir="${build.dir}">
      <manifest>
        <attribute name="Implementation-Version"
                  value="${app.version} build ${build.number}"/>
      </manifest>
    </jar>

  </target>



<!-- ==================== Install Target ================================== -->

<!--

  Copies the application WAR file to ${deploy.dir}.

-->

  <target name="install" depends="dist"
      description="Install application to servlet container">

    <!-- Make sure the deploy directory has been set -->
    <fail unless="deploy.dir" message="deploy.dir not set"/>

    <!-- Copy the application war to the deploy directory -->
    <copy file="${dist.dir}/${app.name}.war" toDir="${deploy.dir}"/>

  </target>


<!-- ==================== Re-install Target =============================== -->

<!--

  The "reinstall" target is a shortcut for "dist", "remove" and "install".

-->

  <target name="reinstall" depends="dist,remove,install"
      description="Re-install application to servlet container"/>


<!-- ==================== Remove Target =================================== -->

<!--

  Deletes the application WAR file from ${deploy.dir}

-->

  <target name="remove"
      description="Remove application from servlet container">

    <!-- Make sure the deploy directory has been set -->
    <fail unless="deploy.dir" message="deploy.dir not set"/>

    <!-- Delete the application war file from the deploy directory -->
    <delete file="${deploy.dir}/${app.name}.war"/>

  </target>


<!-- ==================== Javadoc Target ================================== -->

<!--

  The "javadoc" target creates Javadoc from the application source files. The
  Javadoc is provided as a convenience for developers and it is not included in
  the application WAR file.

-->

  <target name="javadoc" depends="compile"
      description="Create Javadoc API documentation">

    <!-- Create the Javadoc directory as needed -->
    <mkdir dir="${javadoc.dir}/api"/>

    <!-- Create the Javadoc -->
    <javadoc sourcepath="${src.dir}"
                destdir="${javadoc.dir}/api"
           packagenames="*">
      <classpath refid="compile.classpath"/>
    </javadoc>

  </target>

</project>
</#noparse>
