<#-- START TEMPLATE HEADER -->
<#-- category=generic -->
<#-- type=entity -->
<#-- version=1.0 -->
<#-- fileNameTemplate=${f.cla(entity.name)}DAO.java -->
<#-- relativeOutputDirectory=src -->
<#-- relativePackage=data.dao -->
<#-- END TEMPLATE HEADER -->
/*
 * Copyright (c) 2004-2005 Code Canvas Technologies Pty Ltd. All Rights Reserved.
 *
 * Modification and redistribution of this file is permitted subject to the
 * terms and conditions of the RapidJ End User License Agreement:
 * http://www.codecanvas.com.au/rapidj/license.php
 *
 * {{$RapidJ$}}
 */

/*
${f.formatLines(project.copyright, ' * ', 80)}
 */
package ${project.rootPackage}.data.dao;

import java.security.Principal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.log4j.Logger;

import ${project.rootPackage}.data.dso.${f.cla(entity.name)}DSO;
import ${project.rootPackage}.data.to.${f.cla(entity.name)}TO;
import ${project.rootPackage}.util.dao.DAOException;
import ${project.rootPackage}.util.dao.DAOUtils;
import ${project.rootPackage}.util.dao.PagedData;
import ${project.rootPackage}.util.ApplicationException;
import ${project.rootPackage}.util.database.DatabaseUtils;
import ${project.rootPackage}.util.database.SequenceGenerator;
import ${project.rootPackage}.util.database.StatementPreparer;
import ${project.rootPackage}.util.database.StatementPreparerException;

/**
 * The ${f.cla(entity.name)}DAO (${f.cla(entity.name)} Data Access Object)
 * class contains methods for retrieving and persisting ${f.cla(entity.name)}
 * data.
 *
 * It uses the <a href="http://java.sun.com/blueprints/patterns/TransferObject.html">
 * Transfer Object pattern</a> to return or receive data from a calling component.
 *
 * Data Specification Objects (DSOs) can be used to specifiy what data should
 * be loaded into Transfer Objects.
 */
public class ${f.cla(entity.name)}DAO {

	/**
	 * Log4j Logger for this class
	 */
	public static final Logger logger = Logger.getLogger(${f.cla(entity.name)}DAO.class);

	/**
	 * StatementPreparer for this
	 */
	private StatementPreparer statementPreparer;

	/**
	 * Default constructor.
	 */
	public ${f.cla(entity.name)}DAO() throws StatementPreparerException {
		// Create the StatementPreparer used to create PreparedStatements
		statementPreparer = new StatementPreparer();
		// Add the ${entity.name} sql to it
		statementPreparer.addSql("${f.col(entity.name)}-sql.xml", getClass());
	}

	/**
	 * Returns true if the specified ${f.cla(entity.name)} exists.
	 *
	 * @param ${f.att(entity.name)}Id - identifying the ${f.cla(entity.name)}.
	 * @return true if the ${f.cla(entity.name)} exists.
	 */
	public boolean exists${f.cla(entity.name)}(
			Long ${f.att(entity.name)}Id)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;

		// Check the required parameters are present
		if (${f.att(entity.name)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Prepare the statement
			Map values = new HashMap();
			values.put("${f.att(entity.name)}Id", ${f.att(entity.name)}Id);
			ps = statementPreparer.prepare(conn, "count${f.cla(entity.name)}", values);

			// Execute the query
			rs = ps.executeQuery();

			// Get the count
			int count = 0;
			if ((rs!=null) && (rs.next())) {
				count = rs.getInt(1);
			}

			// Process the count
			if (count == 0) {
				// No records
				return false;
			} else {
				return true;
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}

	/**
	 * Returns a ${f.cla(entity.name)}TO containing data for the specified ${f.cla(entity.name)}
	 *
	 * @param ${f.att(entity.name)}Id - identifying the ${f.cla(entity.name)}.
	 * @param ${f.att(entity.name)}DSO - specifying what data to return.
	 * @param returnNull - if true and the ${f.cla(entity.name)} is not found then null
	 * is returned, else if false and not found then a DAOException is thrown.
	 * @return a populated ${f.cla(entity.name)}TO or null if it did not exist.
	 */
	public ${f.cla(entity.name)}TO get${f.cla(entity.name)}(
			Long ${f.att(entity.name)}Id,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO,
			boolean returnNull)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		ArrayList ${f.att(entity.name)}s = null;

		// Check the required parameters are present
		if (${f.att(entity.name)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Prepare the statement
			Map values = new HashMap();
			values.put("${f.att(entity.name)}Id", ${f.att(entity.name)}Id);
			ps = statementPreparer.prepare(conn, "get${f.cla(entity.name)}", values);

			// Execute the query
			rs = ps.executeQuery();

			// Extract ${f.cla(entity.name)}TO's from the ResultSet
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs);

			// Process the results
			if (${f.att(entity.name)}s.isEmpty()) {
				// No results, return null or throw exception
				if (returnNull) {
					return null;
				} else {
					throw new DAOException(
						DAOException.SELECT_BY_ID_RETURNED_NO_RECORDS,
						new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}Id}, null);
				}
			} else if (${f.att(entity.name)}s.size()==1){
				// Found exactly one record.
				${f.cla(entity.name)}TO ${f.att(entity.name)} = (${f.cla(entity.name)}TO) ${f.att(entity.name)}s.get(0);
<#if (entity.children?size > 0)>
				// Populate any children according to the DSO
				populateChildren(${f.att(entity.name)}, ${f.att(entity.name)}DSO);
</#if>
				// Return it
				return  ${f.att(entity.name)};
			} else {
				// More than one record is an error
				throw new DAOException(
					DAOException.SELECT_BY_ID_RETURNED_MULTIPLE_RECORDS,
					new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}Id}, null);
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}
<#list entity.identifyingRelationshipsWithParents as relationship>

	/**
	 * Returns true if the ${f.cla(entity.name)} exists for the specified parent ${f.cla(relationship.parent.name)}
	 *
	 * @param ${f.att(entity.name)}Id - identifying the ${f.cla(entity.name)}.
	 * @param ${f.att(relationship.parentAssociationName)}Id - identifying the parent ${f.cla(relationship.parent.name)}.
	 * @return true if the ${f.cla(entity.name)} exists.
	 */
	public boolean exists${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(entity.name)}Id,
			Long ${f.att(relationship.parentAssociationName)}Id)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;

		// Check the required parameters are present
		if (${f.att(entity.name)}Id==null) throw new NullPointerException();
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Prepare the statement
			Map values = new HashMap();
			values.put("${f.att(entity.name)}Id", ${f.att(entity.name)}Id);
			values.put("${f.att(relationship.parentAssociationName)}Id", ${f.att(relationship.parentAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "count${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}", values);

			// Execute the query
			rs = ps.executeQuery();

			// Get the count
			int count = 0;
			if ((rs!=null) && (rs.next())) {
				count = rs.getInt(1);
			}

			// Process the count
			if (count == 0) {
				// No records
				return false;
			} else {
				return true;
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}

	/**
	 * Returns true if any ${f.cla(entity.name)} exists for the specified parent ${f.cla(relationship.parent.name)}
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id - identifying the parent ${f.cla(relationship.parent.name)}.
	 * @return true if any ${f.cla(entity.name)} exists for the parent.
	 */
	public boolean exists${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;

		// Check the required parameters are present
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Prepare the statement
			Map values = new HashMap();
			values.put("${f.att(relationship.parentAssociationName)}Id", ${f.att(relationship.parentAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "count${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}", values);

			// Execute the query
			rs = ps.executeQuery();

			// Get the count
			int count = 0;
			if ((rs!=null) && (rs.next())) {
				count = rs.getInt(1);
			}

			// Process the count
			if (count == 0) {
				// No records
				return false;
			} else {
				return true;
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}
	<#if relationship.isOneToMany()>

	/**
	 * Returns a list of ${f.cla(entity.name)}TO's that are children of the
	 * specified ${f.cla(relationship.parentAssociationName)}.
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id - identifying the parent ${f.att(relationship.parent.name)}.
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 */
	public ArrayList get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		ArrayList ${f.att(entity.name)}s = null;

		// Check the required parameters are present
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			Map values = new HashMap();
			values.put("${f.att(relationship.parentAssociationName)}Id", ${f.att(relationship.parentAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}", values);

			// Execute the statement
			rs = ps.executeQuery();

			// Extract ${f.cla(entity.name)}TO's from the results
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs);
		<#if (entity.children?size > 0)>

			// Populate children as requested
			populateChildren(${f.att(entity.name)}s, ${f.att(entity.name)}DSO);
		</#if>

			// Return the list
			return ${f.att(entity.name)}s;

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}

	/**
	 * Returns a PagedData of ${f.cla(entity.name)}TO's that are children of the
	 * specified ${f.cla(relationship.parentAssociationName)}.
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id - identifying the parent ${f.att(relationship.parent.name)}.
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 * @param pageNumber - the requested page number.
	 * @param pageSize - the maximum number of records per page.
	 */
	public PagedData get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO,
			int pageNumber,
			int pageSize)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		PagedData ${f.att(entity.name)}s = null;
		int maxPageNumber = 0;

		// Check the required parameters are present
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			HashMap values = new HashMap();
			values.put("${f.att(relationship.parentAssociationName)}Id", ${f.att(relationship.parentAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}", values);

			// Execute the statement
			rs = ps.executeQuery();

			// Extract ${f.cla(entity.name)}TO's from the results
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs, pageNumber, pageSize);
		<#if (entity.children?size > 0)>

			// Populate children as requested
			populateChildren(${f.att(entity.name)}s, ${f.att(entity.name)}DSO);
		</#if>

		} catch (DAOException de) {

			if (de.getKey().equals(DAOException.PAGE_DOES_NOT_EXIST)) {
				// Handle this by requesting the max page number.
				maxPageNumber = ((Integer) de.getArgs()[0]).intValue();
				if (logger.isDebugEnabled()) logger.debug("page " + pageNumber + " does not exist, max page is " + maxPageNumber);
			} else {
				// Throw other DAOExceptions up
				logger.error("dao exception", de);
				throw de;
			}

		} finally {

			// Always close the resources
			DAOUtils.close(rs, ps, conn);

		}

		if (${f.att(entity.name)}s!=null) {
			// Return the paged date
			return ${f.att(entity.name)}s;
		} else {
			// The requested page must not exist. Get the last page.
			return get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(
				${f.att(relationship.parentAssociationName)}Id, ${f.att(entity.name)}DSO, maxPageNumber, pageSize);
		}

	}
<#else>

	/**
	 * Returns a ${f.cla(entity.name)}TO that is the child of the specified ${f.cla(relationship.parentAssociationName)}
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id - specifiying the parent ${f.cla(relationship.parent.name)}.
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 */
	public ${f.cla(entity.name)}TO get${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		ArrayList ${f.att(entity.name)}s = null;

		// Check the required parameters are present
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			HashMap values = new HashMap();
			values.put("${f.att(relationship.parentAssociationName)}Id", ${f.att(relationship.parentAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "get${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}", values);

			// Execute it
			rs = ps.executeQuery();

			// Extract ${f.cla(entity.name)}TO's from the results
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs);

			// Process the results
			if (${f.att(entity.name)}s.size()==0) {
				// No results, return null
				return null;
			} else if (${f.att(entity.name)}s.size()==1){
				// A single result. Populate children and return.
				${f.cla(entity.name)}TO ${f.att(entity.name)} = (${f.cla(entity.name)}TO) ${f.att(entity.name)}s.get(0);
		<#if (entity.children?size > 0)>
				populateChildren(${f.att(entity.name)}, ${f.att(entity.name)}DSO);
		</#if>
				return  ${f.att(entity.name)};
			} else {
				// More than one record is an error
				throw new DAOException(
					DAOException.SELECT_BY_PARENT_ID_RETURNED_MULTIPLE_RECORDS,
					new Object[] {"${f.cla(entity.name)}", "${f.cla(relationship.parentAssociationName)}", ${f.att(relationship.parentAssociationName)}Id}, null);
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}
	</#if>
</#list>
<#list entity.identifyingRelationshipsWithChildren as relationship>
	<#if relationship.isOneToOne()>

	/**
	 * Returns a list of ${f.cla(entity.name)}TO's that are not yet associated
	 * with a ${f.cla(relationship.childAssociationName)}.
	 *
	 * @param ${f.att(entity.name)}Id - the (optional) ${f.cla(entity.name)} to
	 * include in the list regardless.
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 */
	public ArrayList get${f.cla(entity.name)}sWithNo${f.cla(relationship.childAssociationName)}(
			Long ${f.att(entity.name)}Id,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		ArrayList ${f.att(entity.name)}s = null;

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			HashMap values = new HashMap();
			values.put("${f.att(entity.name)}Id", ${f.att(entity.name)}Id);
			ps = statementPreparer.prepare(conn, "get${f.cla(entity.name)}sWithNo${f.cla(relationship.childAssociationName)}", values);

			// Execute the statement
			rs = ps.executeQuery();

			// Extract ${f.cla(entity.name)}TO's from the results
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs);
		<#if (entity.children?size > 0)>

			// Populate children as requested
			populateChildren(${f.att(entity.name)}s, ${f.att(entity.name)}DSO);
		</#if>

			// Return the list
			return ${f.att(entity.name)}s;

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}
	</#if>
</#list>

	/**
	 * Returns a list containing all ${f.cla(entity.name)}TO's.
	 *
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 */
	public ArrayList getAll${f.cla(entity.name)}s(
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		ArrayList ${f.att(entity.name)}s = null;

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			ps = statementPreparer.prepare(conn, "getAll${f.cla(entity.name)}s");

			// Execute it
			rs = ps.executeQuery();

			// Extract the ${f.cla(entity.name)}TO's
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs);
<#if (entity.children?size > 0)>

			// Populate children as requested
			populateChildren(${f.att(entity.name)}s, ${f.att(entity.name)}DSO);
</#if>

			// Return the list
			return ${f.att(entity.name)}s;

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

	}

	/**
	 * Returns a PagedData containing ${f.cla(entity.name)}TO's.
	 *
	 * @param ${f.att(entity.name)}DSO - specifying what data should be loaded.
	 * @param pageNumber - the requested page number.
	 * @param pageSize - the maximum number of records per page.
	 * @exception DAOException if there is a problem.
	 */
	public PagedData getAll${f.cla(entity.name)}s(
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO,
			int pageNumber,
			int pageSize)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;
		ResultSet rs = null;
		PagedData ${f.att(entity.name)}s = null;
		int maxPageNumber = 0;

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			ps = statementPreparer.prepare(conn, "getAll${f.cla(entity.name)}s");

			// Execute it
			rs = ps.executeQuery();

			// Extract the ${f.cla(entity.name)}TO's
			${f.att(entity.name)}s = DAOUtils.extract(${f.cla(entity.name)}TO.class, rs, pageNumber, pageSize);
<#if (entity.children?size > 0)>

			// Populate children as requested
			populateChildren(${f.att(entity.name)}s, ${f.att(entity.name)}DSO);
</#if>

		} catch (DAOException de) {

			if (de.getKey().equals(DAOException.PAGE_DOES_NOT_EXIST)) {
				// Handle this by requesting the max page number.
				maxPageNumber = ((Integer) de.getArgs()[0]).intValue();
				if (logger.isDebugEnabled()) logger.debug("page " + pageNumber + " does not exist, max page is " + maxPageNumber);
			} else {
				// Throw other DAOExceptions up
				logger.error("dao exception", de);
				throw de;
			}

		} finally {

			// Always close resources
			DAOUtils.close(rs, ps, conn);

		}

		if (${f.att(entity.name)}s!=null) {
			// Return the paged date
			return ${f.att(entity.name)}s;
		} else {
			// The requested page must not exist. Get the last page.
			return getAll${f.cla(entity.name)}s(${f.att(entity.name)}DSO, maxPageNumber, pageSize);
		}

	}

	/**
	 * Save the specified ${f.cla(entity.name)}.
	 *
	 * @param ${f.att(entity.name)}TO - containing the data to save.
	 * @param user - the user who is requesting the save.
	 * @return the id of the ${f.cla(entity.name)} that was saved.
	 */
	public Long save${f.cla(entity.name)}(
			${f.cla(entity.name)}TO ${f.att(entity.name)},
			Principal user)
			throws ApplicationException,
				SQLException {

		// Determine if the ${f.att(entity.name)} is a new one
		boolean isNew = ((${f.att(entity.name)}.get${f.cla(entity.name)}Id()==null) ||
			(${f.att(entity.name)}.get${f.cla(entity.name)}Id().longValue()<=0));

		// Get the current time
		Timestamp now = new Timestamp(System.currentTimeMillis());

		// Get the unique indentifier for the user
		String uid = DAOUtils.getUid(user);

		if (isNew) {
			// Get the next unique id using the key sequence generator
			Long nextId = SequenceGenerator.getNextId("${f.att(entity.name)}Id");

			// Setup some initial data values
			${f.att(entity.name)}.set${f.cla(entity.name)}Id(nextId);
			${f.att(entity.name)}.setCreateDatetime(now);
			${f.att(entity.name)}.setCreateUid(uid);
			${f.att(entity.name)}.setVersion(new Long(1));
		}

		// Set the update datetime and updated uid
		${f.att(entity.name)}.setUpdateDatetime(now);
		${f.att(entity.name)}.setUpdateUid(uid);

		// Get the property values from the ${f.cla(entity.name)}TO
		Map values = DAOUtils.getPropertyValues(${f.att(entity.name)});

		PreparedStatement ps = null;
		Connection conn = null;
		int updateCount;

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			if (isNew) {

				// Insert a new ${f.cla(entity.name)}
				// Create the insert statement
				ps = statementPreparer.prepare(conn, "insert${f.cla(entity.name)}", values);
				// And execute it
				updateCount = ps.executeUpdate();

			} else {

				// Update the existing ${f.cla(entity.name)}
				// Create the update statment
				ps = statementPreparer.prepare(conn, "update${f.cla(entity.name)}", values);
				// Execute it
				updateCount = ps.executeUpdate();
				if (updateCount==1) {
					// The update was successful.
					logger.debug("update successful");
				} else if (updateCount==0) {
					// No records were affected.
					if (exists${f.cla(entity.name)}(${f.att(entity.name)}.get${f.cla(entity.name)}Id())) {
						// Record exists, there must have been a concurrent update.
						throw new DAOException(
							DAOException.CONCURRENT_UPDATE,
							new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}.get${f.cla(entity.name)}Id()}, null);
					} else {
						// Record does not exist.
						throw new DAOException(
							DAOException.RECORD_DOES_NOT_EXIST,
							new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}.get${f.cla(entity.name)}Id()}, null);
					}
				} else {
					// Multiple (or negative) number of records affected
					logger.debug("updated " + updateCount + " records");
					throw new DAOException(
						DAOException.UPDATE_AFFECTED_WRONG_NUMBER_RECORDS,
						new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}.get${f.cla(entity.name)}Id()}, null);
				}

			}

		} finally {

			// Always close resources
			DAOUtils.close(ps, conn);

		}

		// Return the id of the ${f.cla(entity.name)} that was saved
		return ${f.att(entity.name)}.get${f.cla(entity.name)}Id();

	}

	/**
	 * Saves all of the supplied ${f.cla(entity.name)}TO's in the list.
	 *
	 * @param ${f.att(entity.name)}s the list of ${f.cla(entity.name)}TO's.
	 * @param user - the user who is requesting the save.
	 * @return an array containing the id's of the saved ${f.cla(entity.name)}TO's.
	 */
	public Long[] save${f.cla(entity.name)}s(
			Collection ${f.att(entity.name)}s,
			Principal user)
			throws ApplicationException,
				SQLException {

		Long[] ${f.att(entity.name)}Ids = new Long[${f.att(entity.name)}s.size()];
		${f.cla(entity.name)}TO ${f.att(entity.name)} = null;
		int i = 0;
		for (Iterator iter = ${f.att(entity.name)}s.iterator(); iter.hasNext();) {
			${f.att(entity.name)} = (${f.cla(entity.name)}TO) iter.next();
			${f.att(entity.name)}Ids[i++] = save${f.cla(entity.name)}(${f.att(entity.name)}, user);
		}
		return ${f.att(entity.name)}Ids;
	}

	/**
	 * Deletes the specified ${f.cla(entity.name)}
	 *
	 * @param ${f.att(entity.name)}Id identifying the ${f.att(entity.name)}Id
	 */
	public void delete${f.cla(entity.name)}(
			Long ${f.att(entity.name)}Id)
			throws ApplicationException,
				SQLException {

		// Check the required parameters are present
		if (${f.att(entity.name)}Id==null) throw new NullPointerException();

<#list entity.relationships as relationship>
	<#if relationship.parent == entity>
		<#-- Child relationships -->
		<#if relationship.cascadeDelete>
			<#-- Cascade delete children that are identified by this record -->

		// Delete ${f.cla(relationship.child.name)}
			<#if relationship.isOneToMany()>
		DAOs.get${f.cla(relationship.child.name)}DAO().delete${f.cla(relationship.child.name)}sFor${f.cla(relationship.parentAssociationName)}(${f.att(entity.name)}Id);
			<#elseif relationship.identifying>
		DAOs.get${f.cla(relationship.child.name)}DAO().delete${f.cla(relationship.child.name)}For${f.cla(relationship.parentAssociationName)}(${f.att(entity.name)}Id);
			</#if>
		<#else>
			<#if relationship.identifying>
				<#-- Check there are no children identified by this record -->

		// Make sure there are no children ${f.cla(relationship.child.name)} records
		if (DAOs.get${f.cla(relationship.child.name)}DAO().exists${f.cla(relationship.child.name)}For${f.cla(relationship.parentAssociationName)}(${f.att(entity.name)}Id)) {
			throw new DAOException(
				DAOException.IDENTIFIED_CHILDREN_EXIST,
				new Object[] {"${f.cla(relationship.child.name)}"}, null);
		}
			</#if>
		</#if>
	<#else>
		<#-- Parent relationships -->
		<#if !relationship.identifying>
		<#-- Clear and parents who have this as a non identified child -->

		// Clear ${f.cla(relationship.parent.name)} reference to this ${f.cla(entity.name)}
		DAOs.get${f.cla(relationship.parent.name)}DAO().clearReferencesTo${f.cla(relationship.childAssociationName)}(${f.att(entity.name)}Id);
		</#if>
	</#if>
</#list>

		// Delete the ${f.cla(entity.name)}
		PreparedStatement ps = null;
		Connection conn = null;

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			HashMap values = new HashMap();
			values.put("${f.att(entity.name)}Id", ${f.att(entity.name)}Id);
			ps = statementPreparer.prepare(conn, "delete${f.cla(entity.name)}", values);

			// Get the number of affected records
			int count = ps.executeUpdate();

			// And make sure a single record was deleted
			if (count == 0) {
				throw new DAOException(
					DAOException.DELETE_BY_ID_AFFECTED_NO_RECORDS,
					new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}Id}, null);
			} else if (count != 1) {
				throw new DAOException(
					DAOException.DELETE_BY_ID_AFFECTED_MULTIPLE_RECORDS,
					new Object[] {"${f.cla(entity.name)}", ${f.att(entity.name)}Id}, null);
			}

		} finally {

			// Always close resources
			DAOUtils.close(ps, conn);

		}

	}
<#list entity.identifyingRelationshipsWithParents as relationship>
	<#if relationship.isOneToMany()>

	/**
	 * Deletes all ${f.cla(entity.name)}s for the specified ${f.cla(relationship.parentAssociationName)}
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id identifying the ${f.cla(relationship.parent.name)}
	 */
	public void delete${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id)
			throws ApplicationException,
				SQLException {

		// Get the list of ${f.cla(entity.name)}s for the specified ${f.cla(relationship.parentAssociationName)}
		ArrayList ${f.att(entity.name)}s = get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(${f.att(relationship.parentAssociationName)}Id);

		// And delete them individually
		for (Iterator iter = ${f.att(entity.name)}s.iterator(); iter.hasNext(); ) {
			${f.cla(entity.name)}TO ${f.att(entity.name)} = (${f.cla(entity.name)}TO) iter.next();
			delete${f.cla(entity.name)}(${f.att(entity.name)}.get${f.cla(entity.name)}Id());
		}

	}
	<#else>

	/**
	 * Deletes the ${f.cla(entity.name)} for the specified ${f.cla(relationship.parentAssociationName)}
	 *
	 * @param ${f.att(relationship.parentAssociationName)}Id identifying the ${f.cla(relationship.parent.name)}
	 * @return false if the ${f.cla(entity.name)} was not found
	 */
	public boolean delete${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(
			Long ${f.att(relationship.parentAssociationName)}Id)
			throws ApplicationException,
				SQLException {

		// Check the required parameters are present
		if (${f.att(relationship.parentAssociationName)}Id==null) throw new NullPointerException();

		// Get the ${f.cla(entity.name)} for the specified ${f.cla(relationship.parentAssociationName)}
		${f.cla(entity.name)}TO ${f.att(entity.name)} = get${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(${f.att(relationship.parentAssociationName)}Id);

		if (${f.att(entity.name)}!=null) {
			// Delete it
			delete${f.cla(entity.name)}(${f.att(entity.name)}.get${f.cla(entity.name)}Id());
			return true;
		} else {
			// Not found
			return false;
		}

	}
	</#if>
</#list>
<#list entity.relationships as relationship>
	<#if relationship.parent == entity && !relationship.identifying>

	/**
	 * Clears any references to the specified ${f.cla(relationship.childAssociationName)}
	 *
	 * @param ${f.att(relationship.childAssociationName)}Id identifying the ${f.cla(relationship.child.name)}
	 */
	public void clearReferencesTo${f.cla(relationship.childAssociationName)}(
			Long ${f.att(relationship.childAssociationName)}Id)
			throws ApplicationException,
				SQLException {

		PreparedStatement ps = null;
		Connection conn = null;

		// Check the required parameters are present
		if (${f.att(relationship.childAssociationName)}Id==null) throw new NullPointerException();

		try {

			// Get a connection
			conn = DatabaseUtils.getConnection();

			// Create the statement
			HashMap values = new HashMap();
			values.put("${f.att(relationship.childAssociationName)}Id", ${f.att(relationship.childAssociationName)}Id);
			ps = statementPreparer.prepare(conn, "clearReferencesTo${f.cla(relationship.childAssociationName)}", values);

			// Get the number of affected records
			int count = ps.executeUpdate();

		} finally {

			// Always close resources
			DAOUtils.close(ps, conn);

		}

	}
	</#if>
</#list>

	// ---- Start Convenience Methods ----

	public ${f.cla(entity.name)}TO get${f.cla(entity.name)}(Long ${f.att(entity.name)}Id) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}(${f.att(entity.name)}Id, new ${f.cla(entity.name)}DSO(), false);
	}

	public ${f.cla(entity.name)}TO get${f.cla(entity.name)}(Long ${f.att(entity.name)}Id, boolean returnNull) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}(${f.att(entity.name)}Id, new ${f.cla(entity.name)}DSO(), returnNull);
	}
<#list entity.identifyingRelationshipsWithParents as relationship>
	<#if relationship.isOneToMany()>

	public ArrayList get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(Long ${f.att(relationship.parentAssociationName)}Id) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(${f.att(relationship.parentAssociationName)}Id, new ${f.cla(entity.name)}DSO());
	}

	public PagedData get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(Long ${f.att(relationship.parentAssociationName)}Id, int pageNumber, int pageSize) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}sFor${f.cla(relationship.parentAssociationName)}(${f.att(relationship.parentAssociationName)}Id, new ${f.cla(entity.name)}DSO(), pageNumber, pageSize);
	}
	<#else>

	public ${f.cla(entity.name)}TO get${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(Long ${f.att(relationship.parentAssociationName)}Id) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}For${f.cla(relationship.parentAssociationName)}(${f.att(relationship.parentAssociationName)}Id, new ${f.cla(entity.name)}DSO());
	}
	</#if>
</#list>
<#list entity.identifyingRelationshipsWithChildren as relationship>
	<#if relationship.isOneToOne()>

	public ArrayList get${f.cla(entity.name)}sWithNo${f.cla(relationship.childAssociationName)}(Long ${f.att(entity.name)}Id) throws ApplicationException, SQLException {
		return get${f.cla(entity.name)}sWithNo${f.cla(relationship.childAssociationName)}(${f.att(entity.name)}Id, new ${f.cla(entity.name)}DSO());
	}
	</#if>
</#list>

	public ArrayList getAll${f.cla(entity.name)}s() throws ApplicationException, SQLException {
		return getAll${f.cla(entity.name)}s(new ${f.cla(entity.name)}DSO());
	}

	public PagedData getAll${f.cla(entity.name)}s(int pageNumber, int pageSize) throws ApplicationException, SQLException {
		return getAll${f.cla(entity.name)}s(new ${f.cla(entity.name)}DSO(), pageNumber, pageSize);
	}

	// ---- End Convenience Methods ----
<#if (entity.children?size > 0)>

	/**
	 * This method populates the children for each ${f.cla(entity.name)} in the specified
	 * list according to the supplied DSO.
	 *
	 * @param ${f.att(entity.name)}s the list of ${f.cla(entity.name)}TO object to populate
	 * @param ${f.att(entity.name)}DSO specifying what children to populate
	 */
	protected void populateChildren(
			ArrayList ${f.att(entity.name)}s,
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {

		for (Iterator iter = ${f.att(entity.name)}s.iterator(); iter.hasNext(); ) {
			${f.cla(entity.name)}TO ${f.att(entity.name)} = (${f.cla(entity.name)}TO) iter.next();
			populateChildren(${f.att(entity.name)}, ${f.att(entity.name)}DSO);
		}
	}

	/**
	 * This method populates the children of the specified ${f.cla(entity.name)}
	 * according to the supplied DSO.
	 *
	 * @param ${f.att(entity.name)} to be populated
	 * @param ${f.att(entity.name)}DSO specifies which children to populate
	 */
	protected void populateChildren(
			${f.cla(entity.name)}TO ${f.att(entity.name)},
			${f.cla(entity.name)}DSO ${f.att(entity.name)}DSO)
			throws ApplicationException,
				SQLException {
	<#list entity.identifyingRelationshipsWithChildren as relationship>
		<#if relationship.isOneToOne()>

		// ${f.cla(relationship.childAssociationName)}
		if (${f.att(entity.name)}DSO.get${f.cla(relationship.childAssociationName)}DSO()!=null) {
			${f.att(entity.name)}.set${f.cla(relationship.childAssociationName)}(DAOs.get${f.cla(relationship.child.name)}DAO().
					get${f.cla(relationship.child.name)}For${f.cla(relationship.parentAssociationName)}(
						${f.att(entity.name)}.get${f.cla(entity.name)}Id(),
						${f.att(entity.name)}DSO.get${f.cla(relationship.childAssociationName)}DSO()));
		}
		<#else>

		// ${f.cla(relationship.childAssociationName)}
		if (${f.att(entity.name)}DSO.get${f.cla(relationship.childAssociationName)}DSO()!=null) {
			${f.att(entity.name)}.set${f.cla(relationship.childAssociationName)}(DAOs.get${f.cla(relationship.child.name)}DAO().
					get${f.cla(relationship.child.name)}sFor${f.cla(relationship.parentAssociationName)}(
						${f.att(entity.name)}.get${f.cla(entity.name)}Id(),
						${f.att(entity.name)}DSO.get${f.cla(relationship.childAssociationName)}DSO()));
		}
		</#if>
	</#list>

	}
</#if>

}
